import es
import time
import os
import popuplib
import cfglib

#plugin information
info = es.AddonInfo() 
info.name     = 'Uptime' 
info.version  = '1.0' 
info.url      = 'http://addons.eventscripts.com/addons/view/uptime' 
info.basename = 'uptime' 
info.author   = 'Errant'

uptime = {}


strings = {}
strings['total'] = "Server has been up for %s"
strings['map'] = "Map has been running for %s"
strings['total_players'] = "%i unique players since server start"
strings['map_players'] = "%i unique players this map"

# CFG SETUP
config = cfglib.AddonCFG(es.getAddonPath("uptime") + "/uptime.cfg")
config.text("******************************")
config.text("  UPTIME SETTINGS")
config.text("******************************")
 
date_format         = config.cvar("uptime_date_format",     0, "Set date format (1 = US format, 0 = EU format)")
log                 = config.cvar("uptime_log",     1, "Log times to file on map start (1=ON, 0=OFF)")
show_round_start    = config.cvar("uptime_rnd_start",     1, "Show uptimes publicly on round start  (1=ON, 0=OFF)")
show_as_popup       = config.cvar("uptime_use_popup",     1, "Show uptimes via popup  (1=ON, 0=OFF)")
 
config.write() # Writes the .cfg to file


def timeInt():
    # return a Epoc seconds integer
    return int(time.time())
    
def strftime(timeS=False):
    # strftime wrapper method
    if int(date_format) == 1:
        format = '[%m/%d/%y %H:%M:%S]'
    else:
        format = '[%d/%m/%y %H:%M:%S]'
    if timeS:
        return time.strftime(format,time.localtime(timeS))
    else:
        return time.strftime(format)
        
def humanize_time(secs):
    # turns the time difference into days, hrs, minutes, secs
    mins, secs = divmod(secs, 60)
    hours, mins = divmod(mins, 60)
    days, hours = divmod(hours, 24)
    return '%02dd %02dh %02dm %02ds' % (days,hours, mins, secs)
   
def show_time(stype="total",user=False,ret=False):
    # show time..
    # self explanatory
    s_timed = humanize_time(timeInt() - uptime[stype])
    if ret == True:
        return s_timed
    if user:
        es.tell(user,'#green',strings["%s" % stype] % s_timed)
    else:
        es.msg('#green',strings["%s" % stype] % s_timed)
        
def show_players(stype="total",user=False,ret=False):
    # show player data
    # self explanatory
    if ret == True:
        return len(uptime["%s_players" % stype])
    if user:
        es.tell(user,'#green',strings["%s_players" % stype] % len(uptime["%s_players" % stype]))
    else:
        es.msg('#green',strings["%s_players" % stype] % len(uptime["%s_players" % stype]))      
def load():
    # reset on load :)
    uptime['total'] = timeInt()
    uptime['map'] = timeInt()
    uptime['total_players'] = []
    uptime['map_players'] = []
    # cfg exec
    config.execute()
    
def es_map_start(ev):
    if int(log) == 1:
        # log this info to file
        fp = open(os.path.join(es.getAddonPath('uptime'),'uptime.log'),'a')
        s_time = strftime()
        fp.write("%s -- Uptime Log, New Map Loaded --\n" % s_time)
        fp.write("%s Map Started: %s \n" % (s_time,strftime(uptime['map'])))
        fp.write("%s Map Time: %s \n" % (s_time,show_time('map',ret=True)))
        fp.write("%s Server Started: %s \n" % (s_time,strftime(uptime['total'])))
        fp.write("%s Total Uptime: %s \n" % (s_time,show_time('total',ret=True)))
        fp.write("%s Map Players: %s \n" % (s_time,show_players('map',ret=True)))
        fp.write("%s Total Players: %s \n" % (s_time,show_players(ret=True)))
        fp.write("%s -- End Update --\n" % s_time)
        fp.close()
    # reset map time & player counts
    uptime['map'] = timeInt()
    uptime['map_players'] = []
    

def player_activate(ev):    
    if ev['es_steamid'] not in uptime['map_players']:
        uptime['map_players'].append(ev['es_steamid'])
    if ev['es_steamid'] not in uptime['total_players']:
        uptime['total_players'].append(ev['es_steamid'])
    
def player_say(ev):
    if ev['text'].startswith('uptime'):
        if int(show_as_popup) == 1:
            popup = popuplib.create('uptime')
            popup.addline('-- Server Uptime --')
            popup.addline(' by Errant')
            popup.addline('>> Server <<')
            popup.addline('Map Started: %s' % strftime(uptime['map']))
            popup.addline('Map Time:    %s' % show_time('map',ret=True))
            popup.addline('Server Started: %s' % strftime(uptime['total']))
            popup.addline('Total Uptime:   %s' % show_time(ret=True))
            popup.addline('>> Players <<')
            popup.addline('Map Players: %i' % show_players('map',ret=True))
            popup.addline('Total Players:   %i' % show_players(ret=True))
            popup.send(ev['userid'])
        else:
            show_time('total',ev['userid'])
            show_time('map',ev['userid'])
            
        
def round_start(ev):
    # show on round start
    if int(show_round_start) == 1:
        show_time()
        show_time('map')
        show_players()
        show_players('map')

